# Analytics Engine

The analysis engine is used to evaluate stored XAPI statements. The results generated in this way can then be integrated into an existing application via the Dashboard SDK.

## Configuring Engines

Analytics Engines are controlled via YAML files, which are stored in the `configuration` directory. The analytics engines scheduler needs to be notified about configuration changes by running `scheduler read-configs`, which reads all existing YAML files and starts or updates analytics engines.

If you are running the analytics engine in a Docker container, you should run the following command.

```bash
$ docker compose exec -it scheduler sh -c 'scheduler read-configs'
```

## API Endpoints

### Retrieve Provider Statements

**Endpoint:**

```
POST /provider/data
```

**Description:** Retrieves provider statements from the LRS (Learning Record Store) based on dynamic filters provided in the request. This endpoint allows analytics engines to query statements efficiently with support for pagination and filtering.

**Request Headers:**

- `Authorization`: `Basic <token>` (Required) - Authentication token.

**Request Parameters (JSON Body):**

| Parameter           | Type    | Description                                                                           |
| ------------------- | ------- | ------------------------------------------------------------------------------------- |
| `last_object_id`    | String  | (Optional) The `_id` from the last retrieved document to paginate results.            |
| `page_size`         | Integer | (Optional) Number of statements to retrieve per request (default: system-configured). |
| `verb.id__in`       | String  | (Optional) Comma-separated list of verb IDs to filter statements.                     |
| `actor.mbox__regex` | String  | (Optional) Regular expression to filter statements based on `actor.mbox`.             |
| `actor.tan__exists` | Boolean | (Optional) Whether to include/exclude statements with `actor.tan`.                    |
| `created_at__gt`    | String  | (Optional) Retrieves statements created after the given timestamp.                    |
| `created_at__lt`    | String  | (Optional) Retrieves statements created before the given timestamp.                   |

**Example Request:**

```json
{
    "last_object_id": "65bfc2a...",
    "page_size": 10,
    "verb.id__in": "run,jump",
    "actor.mbox__regex": "^mailto",
    "actor.tan__exists": false,
    "created_at__gt": "2024-01-01T00:00:00Z"
}
```

**Response:**

```json
{
    "verbs": ["run", "jump"],
    "statements": [
        {
            "_id": "65bfc2a...",
            "actor": {
                "mbox": "mailto:example@example.com"
            },
            "verb": {
                "id": "run"
            },
            "object": {
                "id": "example-object"
            }
        }
    ],
    "page_size": 10
}
```


**Response Codes:**

| Status Code                 | Description                                         |
| --------------------------- | --------------------------------------------------- |
| `200 OK`                    | Successful response with filtered statements.       |
| `400 Bad Request`           | Invalid request parameters.                         |
| `401 Unauthorized`          | Authentication token is missing or invalid.         |
| `500 Internal Server Error` | Server-side error, such as missing provider schema. |
