# Shibboleth 
Shibboleth is a single sign-on login system for computer networks and the Internet. It allows people to sign in using just one identity to various systems run by federations of different organizations or institutions. The federations are often universities or public service organizations.

## General information 
Shibboleth is a web-based technology that implements the HTTP/POST artifact and attribute push profiles of SAML, including both Identity Provider (IdP) and Service Provider (SP) components. Polaris can act as a service provider and import the user from a Shibboleth to allow a single sign-on login.

## Configuration 
To use Polaris together with Shibboleth, several things need to be configured. First, ensure that Polaris is successfully installed and operational. Verify its functionality and stability before proceeding with integration. You should be able to login with the database users.

### Generate Service Provider Information 
Polaris automatically starts a service provider service. You can access the metadata of polaris `https://host/saml2/meta`. The metadata is generated by the public and private key from the configuration file.
Download the file and add the service provider information to your shibboleth system. 

Example of the metadata file:

```xml
<md:EntityDescriptor xmlns:md="urn:oasis:names:tc:SAML:2.0:metadata" validUntil="2043-10-27T21:06:57.986716Z" cacheDuration="P7300D" entityID="https://polaris.digitallearning.gmbh/saml2/meta" ID="ONELOGIN_cfce9f8073759d0b20d49a98dfb2e5b87a69718f">
    <ds:Signature xmlns:ds="http://www.w3.org/2000/09/xmldsig#">
        <ds:SignedInfo>
            <ds:CanonicalizationMethod Algorithm="http://www.w3.org/2001/10/xml-exc-c14n#"/>
            <ds:SignatureMethod Algorithm="http://www.w3.org/2001/04/xmldsig-more#rsa-sha512"/>
            <ds:Reference URI="#ONELOGIN_cfce9f8073759d0b20d49a98dfb2e5b87a69718f">
                <ds:Transforms>
                    <ds:Transform Algorithm="http://www.w3.org/2000/09/xmldsig#enveloped-signature"/>
                    <ds:Transform Algorithm="http://www.w3.org/2001/10/xml-exc-c14n#"/>
                </ds:Transforms>
                <ds:DigestMethod Algorithm="http://www.w3.org/2001/04/xmlenc#sha256"/>
                <ds:DigestValue>Jpeg9HeGiHTAMCvG6jR2Cjmvd8xSZOZbE8SRyGiTgNw=</ds:DigestValue>
            </ds:Reference>
        </ds:SignedInfo>
        <ds:SignatureValue></ds:SignatureValue>
        <ds:KeyInfo>
            <ds:X509Data>
                <ds:X509Certificate></ds:X509Certificate>
            </ds:X509Data>
        </ds:KeyInfo>
    </ds:Signature>
    <md:SPSSODescriptor AuthnRequestsSigned="true" WantAssertionsSigned="true" protocolSupportEnumeration="urn:oasis:names:tc:SAML:2.0:protocol">
        <md:KeyDescriptor use="signing">
            <ds:KeyInfo xmlns:ds="http://www.w3.org/2000/09/xmldsig#">
                <ds:X509Data>
                    <ds:X509Certificate></ds:X509Certificate>
                </ds:X509Data>
            </ds:KeyInfo>
        </md:KeyDescriptor>
        <md:KeyDescriptor use="encryption">
            <ds:KeyInfo xmlns:ds="http://www.w3.org/2000/09/xmldsig#">
                <ds:X509Data>
                    <ds:X509Certificate></ds:X509Certificate>
                </ds:X509Data>
            </ds:KeyInfo>
        </md:KeyDescriptor>
        <md:NameIDFormat>urn:oasis:names:tc:SAML:2.0:nameid-format:transient</md:NameIDFormat>
        <md:AssertionConsumerService Binding="urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST" Location="https://polaris.digitallearning.gmbh/saml2/acs" index="1"/>
    </md:SPSSODescriptor>
    <md:Organization>
        <md:OrganizationName xml:lang="de">Polaris Rights Engine</md:OrganizationName>
        <md:OrganizationName xml:lang="en">Polaris Rights Engine</md:OrganizationName>
        <md:OrganizationDisplayName xml:lang="de">Polaris Rights Engine</md:OrganizationDisplayName>
        <md:OrganizationDisplayName xml:lang="en">Polaris Rights Engine</md:OrganizationDisplayName>
        <md:OrganizationURL xml:lang="de">https://www.digitallearning.gmbh</md:OrganizationURL>
        <md:OrganizationURL xml:lang="en">https://www.digitallearning.gmbh</md:OrganizationURL>
    </md:Organization>
    <md:ContactPerson contactType="technical">
        <md:GivenName>Service Desk</md:GivenName>
        <md:EmailAddress>mailto:support@digitallearning.gmbh</md:EmailAddress>
    </md:ContactPerson>
    <md:ContactPerson contactType="support">
        <md:GivenName>Service Desk</md:GivenName>
        <md:EmailAddress>mailto:support@digitallearning.gmbh</md:EmailAddress>
    </md:ContactPerson>
    <md:ContactPerson contactType="administrative">
        <md:GivenName>Service Desk</md:GivenName>
        <md:EmailAddress>mailto:support@digitallearning.gmbh</md:EmailAddress>
    </md:ContactPerson>
</md:EntityDescriptor>
```

### Add Shibboleth meta information

Next, we'll incorporate the metadata pertaining to the Identity Provider (IdP) into Polaris. The IdP generates an XML file that needs to be integrated into Polaris. Notably, Polaris comes preloaded with a sample IdP information file. This generated XML file from the IdP needs to be appended or replaced within Polaris to establish the necessary connection and authentication protocols.

The file looks like the following example:

```xml
<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<EntityDescriptor entityID="https://aai-test-v3.ruhr-uni-bochum.de/idp/shibboleth"  xmlns="urn:oasis:names:tc:SAML:2.0:metadata" xmlns:ds="http://www.w3.org/2000/09/xmldsig#" xmlns:idpdisc="urn:oasis:names:tc:SAML:profiles:SSO:idp-discovery-protocol" xmlns:init="urn:oasis:names:tc:SAML:profiles:SSO:request-init" xmlns:mdattr="urn:oasis:names:tc:SAML:metadata:attribute" xmlns:mdrpi="urn:oasis:names:tc:SAML:metadata:rpi" xmlns:mdui="urn:oasis:names:tc:SAML:metadata:ui" xmlns:remd="http://refeds.org/metadata" xmlns:saml="urn:oasis:names:tc:SAML:2.0:assertion" xmlns:saml1md="urn:mace:shibboleth:metadata:1.0" ID="DFN-AAI-Test_20230208221256" Name="https://www.aai.dfn.de/DFN-AAI-Test" validUntil="2023-02-14T22:12:56Z">
    <Extensions>
        <mdrpi:RegistrationInfo registrationAuthority="https://www.aai.dfn.de" registrationInstant="2016-09-15T07:04:09Z">
            <mdrpi:RegistrationPolicy xml:lang="de">https://www.aai.dfn.de/teilnahme/</mdrpi:RegistrationPolicy>
            <mdrpi:RegistrationPolicy xml:lang="en">https://www.aai.dfn.de/en/join/</mdrpi:RegistrationPolicy>
        </mdrpi:RegistrationInfo>
    </Extensions>
    <IDPSSODescriptor protocolSupportEnumeration="urn:oasis:names:tc:SAML:2.0:protocol">
        <Extensions>
            <saml1md:Scope regexp="false">ruhr-uni-bochum.de</saml1md:Scope>
            <mdui:UIInfo>
                <mdui:DisplayName xml:lang="de">Ruhr-Universität Bochum TEST-IDP-v3</mdui:DisplayName>
                <mdui:DisplayName xml:lang="en">Ruhr-Universitaet Bochum TEST-IDP-v3</mdui:DisplayName>
                <mdui:Description xml:lang="de">Test-IdP der Ruhr-Universität Bochum für die Shibboleth v3 Migration</mdui:Description>
                <mdui:Description xml:lang="en">Ruhr-Universitaet Bochum Test-IdP for Shibboleth v3 migration</mdui:Description>
                <mdui:Logo height="102" width="102">https://aai.ruhr-uni-bochum.de/idp/images/logo/logo-rub-102.gif</mdui:Logo>
                <mdui:Logo height="16" width="16">https://www.ruhr-uni-bochum.de/themes/custom/rub/favicon.ico</mdui:Logo>
            </mdui:UIInfo>
        </Extensions>
        <KeyDescriptor>
            <ds:KeyInfo>
                <ds:KeyName>aai-test-v3.ruhr-uni-bochum.de</ds:KeyName>
                <ds:X509Data>
                    <ds:X509SubjectName></ds:X509SubjectName>
                    <ds:X509Certificate></ds:X509Certificate>
                </ds:X509Data>
            </ds:KeyInfo>
        </KeyDescriptor>
        <ArtifactResolutionService Binding="urn:oasis:names:tc:SAML:2.0:bindings:SOAP" Location="https://aai-test-v3.ruhr-uni-bochum.de/idp/profile/SAML2/SOAP/ArtifactResolution" index="2"/>
        <SingleLogoutService Binding="urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST" Location="https://aai-test-v3.ruhr-uni-bochum.de/idp/profile/SAML2/POST/SLO"/>
        <SingleLogoutService Binding="urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST-SimpleSign" Location="https://aai-test-v3.ruhr-uni-bochum.de/idp/profile/SAML2/POST-SimpleSign/SLO"/>
        <SingleLogoutService Binding="urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect" Location="https://aai-test-v3.ruhr-uni-bochum.de/idp/profile/SAML2/Redirect/SLO"/>
        <SingleLogoutService Binding="urn:oasis:names:tc:SAML:2.0:bindings:SOAP" Location="https://aai-test-v3.ruhr-uni-bochum.de/idp/profile/SAML2/SOAP/SLO"/>
        <NameIDFormat>urn:oasis:names:tc:SAML:2.0:nameid-format:transient</NameIDFormat>
        <SingleSignOnService Binding="urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST" Location="https://aai-test-v3.ruhr-uni-bochum.de/idp/profile/SAML2/POST/SSO"/>
        <SingleSignOnService Binding="urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST-SimpleSign" Location="https://aai-test-v3.ruhr-uni-bochum.de/idp/profile/SAML2/POST-SimpleSign/SSO"/>
        <SingleSignOnService Binding="urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect" Location="https://aai-test-v3.ruhr-uni-bochum.de/idp/profile/SAML2/Redirect/SSO"/>
    </IDPSSODescriptor>
    <AttributeAuthorityDescriptor protocolSupportEnumeration="urn:oasis:names:tc:SAML:2.0:protocol">
        <Extensions>
            <saml1md:Scope regexp="false">ruhr-uni-bochum.de</saml1md:Scope>
        </Extensions>
        <KeyDescriptor>
            <ds:KeyInfo>
                <ds:KeyName>aai-test-v3.ruhr-uni-bochum.de</ds:KeyName>
                <ds:X509Data>
                    <ds:X509SubjectName></ds:X509SubjectName>
                    <ds:X509Certificate></ds:X509Certificate>
                </ds:X509Data>
            </ds:KeyInfo>
        </KeyDescriptor>
        <AttributeService Binding="urn:oasis:names:tc:SAML:2.0:bindings:SOAP" Location="https://aai-test-v3.ruhr-uni-bochum.de/idp/profile/SAML2/SOAP/AttributeQuery"/>
    </AttributeAuthorityDescriptor>
    <Organization>
        <OrganizationName xml:lang="de">e200</OrganizationName>
        <OrganizationName xml:lang="en">e200</OrganizationName>
        <OrganizationDisplayName xml:lang="de">Ruhr-Universität Bochum</OrganizationDisplayName>
        <OrganizationDisplayName xml:lang="en">Ruhr-Universität Bochum</OrganizationDisplayName>
        <OrganizationURL xml:lang="de">http://www.ruhr-uni-bochum.de</OrganizationURL>
        <OrganizationURL xml:lang="en">http://www.ruhr-uni-bochum.de</OrganizationURL>
    </Organization>
    <ContactPerson contactType="other" remd:contactType="http://refeds.org/metadata/contactType/security">
        <GivenName>AAI</GivenName>
        <SurName>RUB</SurName>
        <EmailAddress>mailto:aai@ruhr-uni-bochum.de</EmailAddress>
    </ContactPerson>
    <ContactPerson contactType="support">
        <GivenName>AAI</GivenName>
        <SurName>RUB</SurName>
        <EmailAddress>mailto:aai@ruhr-uni-bochum.de</EmailAddress>
    </ContactPerson>
    <ContactPerson contactType="technical">
        <GivenName></GivenName>
        <SurName></SurName>
        <EmailAddress></EmailAddress>
    </ContactPerson>
</EntityDescriptor>
```

The file must be placed in the static folder of Polaris. If you use docker, you can mount the file using the volume pattern, e.g.:

```json
    volumes:
      - "./shibboleth.xml:/usr/src/app/static/shibboleth.xml"
```


### Configure additional attributes 
You might need to set up the attribute mapping for the Shibboleth user. This involves configuring options by overriding the default values for the following attributes:

```json
SSO_USERNAME=urn:oid:0.9.2342.19200300.100.1.1 # uid
SSO_EMAIL=urn:oid:0.9.2342.19200300.100.1.3 # mail
SSO_FORENAME=urn:oid:2.16.840.1.113730.3.1.241  # givenName
SSO_SURNAME=urn:oid:2.16.840.1.113730.3.1.241 # sn
SSO_DISPLAY_NAME=urn:oid:2.16.840.1.113730.3.1.241  # displayName
```

### Debug shibboleth configuration

We've introduced a debug page specifically designed to troubleshoot issues related to Shibboleth or other SAML2 Identity Provider (IdP) systems. Access this page via the following link: `https://host/sso-dev`

We strongly advise hiding this page in a production environment to prevent unauthorized access or potential security vulnerabilities.


## Shibboleth ID-Connector
If you have various user identifiers across your provider systems, you can enable a service within Polaris and Shibboleth. This service ensures that you consistently maintain the same identifier in the xAPI statements. When the service is active, Polaris will exclusively accept usernames in the following format: `lms-id@lms-id-type`, e.g. `3249234234@mnr`. The software will separate the ID and its type, then establish a connection with Shibboleth to retrieve the Polaris-ID. This Polaris-ID serves as the unique identifier within Polaris and will substitute the actor in the xAPI statement.

### Flow to resolve the user

![](img/shibboleth_connector_flow.drawio.png "Flow to resolve the user using Shibboleth ID-Connector")

### Base configuration
The ID-Connector can be configured using the following `.env` variables:

```json
SHIB_ID_CONNECTOR_ENABLED=true # feature flag
SHIB_ID_CONNECTOR_CACHE=false # polaris can cache request of identifier and type, so that shibboleth will only asked if the key is missing in the cache
SHIB_ID_CONNECTOR_CACHE_MAX_AGE= # max age of the keys in the cache in seconds
SHIB_ID_CONNECTOR_URL= # url of the service 
SHIB_ID_CONNECTOR_APP_SECRET= # app secret between shib and polaris
SHIB_ID_CONNECTOR_PROCESS_ID= # string for the polaris id, default is 'PI', please ask your shibboleth provider for more information
SHIB_ID_CONNECTOR_LINK_TYPE= # can be leaved empty, please ask your shibboleth provider for more information
```

### Mapping based on file
If it's not possible to modify the provider system, an alternative method within Polaris can be employed to accommodate different IDs across various providers. As a solution, integrating a JSON-based mapping table into Polaris becomes essential. This table would store comprehensive details about the distinct identifiers used within each provider's system. For example:

```json
{
    "moodle-bochum" : "lms-id",
    "moodle-aachen" : "email",
    "providername" : "second-id"    
}
```
In Polaris, the key corresponds to the provider's name, while the value represents the identifier type of Shibboleth.
Enable the feature flag:

```json
SHIB_ID_CONNECTOR_USE_FILE_MAPPING=true # feature flag
```

Remember to mount the file using a Docker volume into Polaris. This ensures that the JSON mapping file remains accessible within the Polaris container.

```json
    volumes:
      - "./file_on_host_name.json:/usr/src/app/static/shibboleth_mapping.json"
```

### Third-Party-Access and Shibboleth ID-Connector
You can seamlessly integrate the Shibboleth ID-Connector with Third-Party Access by simply adjusting the email parameter in the respective endpoints to the updated format `lms-id@lms-id-type`. This adjustment doesn't necessitate any additional requirements or configurations. If you're utilizing file-based mapping, Polaris will automatically recognize the identifier type based on the mappings outlined in the JSON file. This streamlined approach enables a smooth combination of functionalities without the need for extra setup.