"""
Django settings for backend project.

Generated by 'django-admin startproject' using Django 4.1.2.

For more information on this file, see
https://docs.djangoproject.com/en/4.1/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/4.1/ref/settings/
"""
import mimetypes
import os
from pathlib import Path

import environ
from django import urls

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent

# Take environment variables from .env file
env = environ.Env()
environ.Env.read_env()

mimetypes.add_type("text/css", ".css", True)
mimetypes.add_type("text/javascript", ".js", True)


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/4.1/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "django-insecure-4%wn*pt3vpdgk#==(qbtocdh#-2wr58py(ri=(0a^15&&n*gl9"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = env("DEBUG", default=False) == True
DEBUG_PROPAGATE_EXCEPTIONS = env("DEBUG_PROPAGATE_EXCEPTIONS", default=False) == True

TESTING = env("TESTING", default="False")

ALLOW_ROBOTS = env("ALLOW_ROBOTS", default=False)

ANALYTICS_RESULTS_RETENTION = env("ANALYTICS_RESULTS_RETENTION", default=31)
ALLOWED_HOSTS = ["*"]

IDP_ENABLED = env("IDP_ENABLED")
SHOW_XAPI_STATEMENTS = env("SHOW_XAPI_STATEMENTS", default=False)

DISABLE_XAPI_CONSENT = env("DISABLE_XAPI_CONSENT",default=False)

# Application definition

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
   # "django.contrib.staticfiles",
    "corsheaders",
    "rest_framework",
    "rolepermissions",
    "django_celery_results",
    "django_celery_beat",
    "backend",
    "xapi.apps.XapiConfig",
    "users.apps.UsersConfig",
    "consents.apps.ConsentsConfig",
    "providers.apps.ProvidersConfig",
    "data_disclosure.apps.DataDisclosureConfig",
    "data_removal.apps.DataRemovalConfig",
    "drf_yasg",
    "ssoauth"
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",   
    "whitenoise.middleware.WhiteNoiseMiddleware", # Here
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "corsheaders.middleware.CorsMiddleware",
]

ROOT_URLCONF = "backend.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": ["email_templates",os.path.join(BASE_DIR, "frontend/dist/frontend")],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

WSGI_APPLICATION = "backend.wsgi.application"


# Database
# https://docs.djangoproject.com/en/4.1/ref/settings/#databases

if env("TESTING", default="False") == "True":
    DATABASES = {
        "default": {
            "ENGINE": "django.db.backends.sqlite3",
            "NAME": BASE_DIR / "db.sqlite3",
        }
    }
else:
    DATABASES = {
        "default": {
            "ENGINE": "django.db.backends.mysql",
            "NAME": env("DATABASE_NAME"),
            "USER": env("DATABASE_USER"),
            "PASSWORD": env("DATABASE_PASSWORD"),
            "HOST": env("DATABASE_HOST"),
            "PORT": "3306",
        }
    }

# Cache(s)
# https://docs.djangoproject.com/en/4.1/topics/cache/
CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.redis.RedisCache' if env("CACHE_BACKEND", default="file") == "redis"
                    else 'django.core.cache.backends.filebased.FileBasedCache',
        'LOCATION': '/tmp/django_cache' if env("CACHE_BACKEND", default="file") == 'file' else env("CACHE_URI", default='redis://127.0.0.1:6379') ,
    }
}

# Password validation
# https://docs.djangoproject.com/en/4.1/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]


# Internationalization
# https://docs.djangoproject.com/en/4.1/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/4.1/howto/static-files/

STATIC_URL = "/app/de/"
STATICFILES_DIRS = [
  #  BASE_DIR / "static",
     os.path.join(BASE_DIR,  "static"),
     os.path.join(BASE_DIR,  "frontend/dist/frontend/de"),
]

# Default primary key field type
# https://docs.djangoproject.com/en/4.1/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

CORS_ORIGIN_ALLOW_ALL = True

AUTH_USER_MODEL = "users.CustomUser"
REST_FRAMEWORK = {
    "DEFAULT_AUTHENTICATION_CLASSES": (
        "rest_framework_simplejwt.authentication.JWTAuthentication",
    ),
    'DEFAULT_THROTTLE_CLASSES': [
        'rest_framework.throttling.ScopedRateThrottle',
    ],
    'DEFAULT_THROTTLE_RATES': {
        'merge': '3/hour',
    }
}


ROLEPERMISSIONS_MODULE = "backend.roles"

# Number of days after which a user data disclosure expires
DATA_DISCLOSURE_EXPIRATION = env("DATA_DISCLOSURE_EXPIRATION")
DATA_DISCLOSURE_LOCATION = env("DATA_DISCLOSURE_LOCATION")


# Email
EMAIL_BACKEND = "django.core.mail.backends.smtp.EmailBackend"
EMAIL_HOST = env("EMAIL_HOST")
EMAIL_PORT = env("EMAIL_PORT")
EMAIL_USE_TLS = True
EMAIL_HOST_EMAIL = env("EMAIL_HOST_EMAIL", default=env("EMAIL_HOST_USER"))
EMAIL_HOST_USER = env("EMAIL_HOST_USER")
EMAIL_HOST_EMAIL = env("EMAIL_HOST_EMAIL", default=EMAIL_HOST_USER)
EMAIL_HOST_PASSWORD = env("EMAIL_HOST_PASSWORD")



CELERY_BROKER_URL = env("CELERY_BROKER_URL")
CELERY_TASK_ALWAYS_EAGER = env("CELERY_TASK_ALWAYS_EAGER", default=False) 
CELERY_TASK_EAGER_PROPAGATES = env("CELERY_TASK_EAGER_PROPAGATES", default=False)
CELERY_ACCEPT_CONTENT = ['json']  # Only allow JSON serialization for tasks
CELERY_TASK_SERIALIZER = 'json'  # Serialize tasks in JSON
CELERY_RESULT_BACKEND = CELERY_BROKER_URL  # Use the same broker as the result backend (optional)
CELERY_TIMEZONE = TIME_ZONE  # Align Celery's timezone with Django's timezone

# Learning Record Store (LRS) MongoDB connection string
LRS_CONNECTION_STRING = env("LRS_CONNECTION_STRING")
LRS_MONGO_DB_NAME = env("LRS_MONGO_DB_NAME")
ANONYMIZATION_DEFAULT_MINIMUM_COUNT = env("ANONYMIZATION_DEFAULT_MINIMUM_COUNT", default=3)
ANONYMIZATION_HASH_PREFIX = env("ANONYMIZATION_HASH_PREFIX", default="polaris-any-")

JWT_AUTH = {
    "JWT_PUBLIC_KEY": open(env("JWT_PUBLIC_KEY_PATH")).read(),
    "JWT_PRIVATE_KEY": open(env("JWT_PRIVATE_KEY_PATH")).read(),
    "JWT_ALGORITHM": "RS256",
    "JWT_EXPIRATION_DELTA_MINUTES": 30,
}
SESSION_COOKIE_AGE = 36000 
SESSION_EXPIRE_AT_BROWSER_CLOSE = True 


# SSO 
SP_HOST = env("SP_HOST",default="")  # FQDN of your SP
IDP_META_URL = env("IDP_SERVER",default="") + "/idp/shibboleth"  # production
IDP_LOGOUT_URL = env("IDP_LOGOUT_URL", default=env("IDP_SERVER",default="") + "/idp/profile/Logout")  # web page for IDP logout (might initiate SLO)

ATTRIBUTE_USERNAME = env("SSO_USERNAME",default="urn:oid:0.9.2342.19200300.100.1.1")  # "uid"
ATTRIBUTE_EMAIL = env("SSO_EMAIL",default="urn:oid:0.9.2342.19200300.100.1.3")  # "mail"
ATTRIBUTE_FORENAME = env("SSO_FORENAME",default="urn:oid:2.16.840.1.113730.3.1.241")  # "givenName"
ATTRIBUTE_SURNAME = env("SSO_SURNAME",default="urn:oid:2.16.840.1.113730.3.1.241")  # "sn"
ATTRIBUTE_DISPLAY_NAME = env("SSO_DISPLAY_NAME",default="urn:oid:2.16.840.1.113730.3.1.241") # displayName
ATTRIBUTE_ACCOUNT_UUID = env("SSO_ACCOUNT_UUID",default=None) # default not required 
ATTRIBUTE_PAIRWISE_ID = env("SSO_PAIRWISE_ID",default="urn:oasis:names:tc:SAML:attribute:pairwise-id")

SHIB_ID_CONNECTOR_ENABLED = env("SHIB_ID_CONNECTOR_ENABLED", default=False)
SHIB_ID_CONNECTOR_CACHE = env("SHIB_ID_CONNECTOR_CACHE", default=True)
SHIB_ID_CONNECTOR_CACHE_MAX_AGE = env("SHIB_ID_CONNECTOR_CACHE_MAX_AGE", default=24*60)
SHIB_ID_CONNECTOR_URL = env("SHIB_ID_CONNECTOR_URL",default="")
SHIB_ID_CONNECTOR_APP_SECRET = env("SHIB_ID_CONNECTOR_APP_SECRET",default="")
SHIB_ID_CONNECTOR_PROCESS_ID = env("SHIB_ID_CONNECTOR_PROCESS_ID",default="PI")
SHIB_ID_CONNECTOR_LINK_TYPE = env("SHIB_ID_CONNECTOR_LINK_TYPE",default="")
SHIB_ID_CONNECTOR_USE_FILE_MAPPING = env("SHIB_ID_CONNECTOR_USE_FILE_MAPPING",default=False)