import { Component, HostBinding, OnDestroy, OnInit } from '@angular/core'
import { LoadingService } from './services/loading.service'
import { delay, Subscription } from 'rxjs'
import { ThemeService } from './services/theme.service'
import { AuthService, Userinfo } from './services/auth.service'
import { WizardDialog } from './consent-management/wizard/wizard.component'
import { ApiService, UserConsentStatus } from './services/api.service'
import { Router } from '@angular/router'
import { NzModalRef, NzModalService } from 'ng-zorro-antd/modal'

@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.scss']
})
export class AppComponent implements OnInit, OnDestroy {
  @HostBinding('class') className = ''
  title = 'frontend'
  loading = false
  isDarkModeOn: boolean = false
  loggedIn: Userinfo | null = null
  dialogCloseSubscription?: Subscription
  wizardDialogRef: NzModalRef<WizardDialog> | null = null

  constructor(
    private _loading: LoadingService,
    private _theme: ThemeService,
    private _authService: AuthService,
    private _dialog: NzModalService,
    private _apiService: ApiService,
    private _router: Router
  ) {
    this._theme.darkModeSubject.subscribe((x) => (this.isDarkModeOn = x))
  }

  ngOnInit() {
    this.listenToLoading()
    this._authService.loggedIn.subscribe((userInfo) => {
      if (userInfo && !userInfo.isProvider) {
        this._apiService.getUserConsentStatus().subscribe((userConsent) => {
          if (Object.values(userConsent.result).length === 0) this.openWizard(null)
          const isAnyOutdatedOrNone = !!Object.values(userConsent.result).find(
            ({ status }) => status === 'outdated' || status === 'none'
          )
          if (isAnyOutdatedOrNone) this.openWizard(userConsent)
        })
      }
      this.loggedIn = userInfo
    })
  }

  ngOnDestroy(): void {
    this.dialogCloseSubscription?.unsubscribe()
  }

  listenToLoading(): void {
    this._loading.loadingSub
      .pipe(delay(0)) // This prevents a ExpressionChangedAfterItHasBeenCheckedError for subsequent requests
      .subscribe((loading) => {
        this.loading = loading
      })
  }

  openWizard(userConsentStatus: UserConsentStatus | null): void {
    if (!this.wizardDialogRef) {
      this.wizardDialogRef = this._dialog.create({
        nzContent: WizardDialog,
        nzMaskClosable: false,
        nzClosable: false,
        nzWidth: '90vw',
        nzComponentParams: {
          data: userConsentStatus
        }
      })

      this.dialogCloseSubscription = this.wizardDialogRef.afterClose.subscribe((result) => {
        this._router
          .navigateByUrl('/', { skipLocationChange: true })
          .then(() => this._router.navigate(['/consent-management']))
      })
    }
  }
}
