import { Component } from '@angular/core'
import { NzMessageService } from 'ng-zorro-antd/message'
import { NzModalService } from 'ng-zorro-antd/modal'
import { ApiService, AuthGroup, AuthPermission, AuthUser } from '../services/api.service'

interface AuthGroupCheckbox {
  label: string
  value: string
  checked?: boolean
}
type UserGroupCheckboxMap = Record<number, AuthGroupCheckbox[]>; // id to checkboxes

@Component({
  selector: 'app-control-center',
  templateUrl: './control-center.component.html',
  styleUrls: ['./control-center.component.scss']
})

export class ControlCenterComponent {
  protected users: AuthUser[] = [];
  protected auth_permissions: AuthPermission[] = [];
  protected auth_groups: AuthGroup[] = [];

  expandSet = new Set<number>();
  protected user_group_checkboxes: UserGroupCheckboxMap = {}
  protected user_permission_checkboxes: UserGroupCheckboxMap = {}

  onExpandChange(id: number, checked: boolean): void {
    if (checked) {
      this.expandSet.clear();
      this.expandSet.add(id);
    } else {
      this.expandSet.delete(id);
    }
  }
  constructor(
    private _messageService: NzMessageService,
    private _apiService: ApiService,
    public dialog: NzModalService
  ) {

    this._apiService.getGroups().subscribe(result => {
      this.auth_groups = result;
      this._apiService.getPermissions().subscribe(result => {
        this.auth_permissions = result;
        this._apiService.getUsers().subscribe(result => {
          this.users = result;
          let new_group_checkboxes: UserGroupCheckboxMap = {};
          let new_permission_checkboxes: UserGroupCheckboxMap = {};
          result.forEach((user) => {
            new_group_checkboxes[user.id] = this.auth_groups.map((group) => {
              return {
                label: group.name,
                value: group.id.toString(),
                checked: user.groups.map(gr => gr.id).includes(group.id)
              }
            })
            new_permission_checkboxes[user.id] = this.auth_permissions.map((permission) => {
              return {
                label: permission.name,
                value: permission.id.toString(),
                checked: user.user_permissions.map(pr => pr.id).includes(permission.id)
              }
            })

          })
          this.user_group_checkboxes = new_group_checkboxes;
          this.user_permission_checkboxes = new_permission_checkboxes;
        })
      })
    })
  }

}
